@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -37,7 +37,7 @@
 
 #include "body_stop_ball.h"
 //#include "body_kick_to_relative.h"
-#include "body_hold_ball.h"
+#include "body_hold_ball2008.h"
 
 #include <rcsc/player/player_agent.h>
 #include <rcsc/common/logger.h>
@@ -120,10 +120,10 @@ Body_KickOneStep::execute( PlayerAgent * agent )
                       __FILE__": why power over??  power=%f", kick_power );
         if ( ! M_force_mode )
         {
-            return Body_HoldBall( true,
-                                  M_target_point,
-                                  M_target_point
-                                  ).execute( agent );
+            return Body_HoldBall2008( true,
+                                      M_target_point,
+                                      M_target_point
+                                      ).execute( agent );
         }
         kick_power = ServerParam::i().maxPower();
     }
@@ -168,9 +168,16 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
     if ( num == 0 )
     {
         dlog.addText( Logger::KICK,
-                      __FILE__": get_max_possible_vel: angle=%.1f. No solution",
+                      __FILE__": (get_max_possible_vel) angle=%.1f. No solution. try to stop the ball",
                       target_angle.degree() );
-        return Vector2D( 0.0, 0.0 );
+        //return Vector2D( 0.0, 0.0 );
+        Vector2D accel = -ball_vel;
+        double accel_r = accel.r();
+        if ( accel_r > max_accel )
+        {
+            accel *= max_accel / accel_r;
+        }
+        return accel; // stop the ball
     }
 
     if ( num == 1 )
@@ -184,7 +191,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
                 sol1.setLength( ServerParam::i().ballSpeedMax() );
 
                 dlog.addText( Logger::KICK,
-                              __FILE__": get_max_possible_vel: angle=%.1f."
+                              __FILE__": (get_max_possible_vel) angle=%.1f."
                               " 1 solution  adjust.",
                               target_angle.degree() );
             }
@@ -194,7 +201,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
                 sol1.assign( 0.0, 0.0 );
 
                 dlog.addText( Logger::KICK,
-                              __FILE__": get_max_possible_vel: angle=%.1f."
+                              __FILE__": (get_max_possible_vel) angle=%.1f."
                               " 1 solution. failed.",
                               target_angle.degree() );
             }
@@ -219,7 +226,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
         std::swap( sol1, sol2 );
         std::swap( length1, length2 );
         dlog.addText( Logger::KICK,
-                      __FILE__": get_max_possible_vel: swap" );
+                      __FILE__": (get_max_possible_vel) swap" );
     }
 
     if ( length1 > ServerParam::i().ballSpeedMax() )
@@ -229,7 +236,7 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
             sol1.assign( 0.0, 0.0 );
 
             dlog.addText( Logger::KICK,
-                          __FILE__": get_max_possible_vel: angle=%.1f."
+                          __FILE__": (get_max_possible_vel) angle=%.1f."
                           " 2 solutions. but never reach",
                           target_angle.degree() );
         }
@@ -238,14 +245,14 @@ Body_KickOneStep::get_max_possible_vel( const AngleDeg & target_angle,
             sol1.setLength( ServerParam::i().ballSpeedMax() );
 
             dlog.addText( Logger::KICK,
-                          __FILE__": get_max_possible_vel: angle=%.1f."
+                          __FILE__": (get_max_possible_vel) angle=%.1f."
                           " 2 solutions. adjust to ballSpeedMax",
                           target_angle.degree() );
         }
     }
 
     dlog.addText( Logger::KICK,
-                  __FILE__": one kick: 2 solutions: angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
+                  __FILE__": (get_max_possible_vel) 2 solutions: angle=%.1f max_vel=(%.2f, %.2f)r=%.2f",
                   target_angle.degree(),
                   sol1.x, sol1.y, sol1.r() );
 
