@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,9 +41,8 @@
 //#include "body_kick_multi_step.h"
 #include "body_smart_kick.h"
 #include "body_stop_ball.h"
-#include "body_hold_ball.h"
+#include "body_hold_ball2008.h"
 
-#include <rcsc/player/interception.h>
 #include <rcsc/player/player_agent.h>
 #include <rcsc/player/debug_client.h>
 #include <rcsc/player/audio_sensor.h>
@@ -1065,25 +1064,6 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
                   first_speed, target_angle.degree() );
 #endif
 
-#if 0
-    const Interception interception( world.ball().pos() + first_vel,
-                                     first_speed * ServerParam::i().ballDecay(),
-                                     target_angle );
-    {
-        int cycle = static_cast< int >
-            ( std::ceil( interception.getReachCycle( receiver->pos(),
-                                                     &(receiver->vel()),
-                                                     ( receiver->bodyCount() == 0
-                                                       ? &(receiver->body()) : NULL ),
-                                                     receiver->posCount(),
-                                                     0.9, // kickable area + buf
-                                                     1.0 ) ) ); // dash speed
-        dlog.addText( Logger::PASS,
-                      "______ receiver reach cycle by util = %d.",
-                      cycle );
-    }
-#endif
-
     const PlayerPtrCont::const_iterator
         o_end = world.opponentsFromSelf().end();
     for ( PlayerPtrCont::const_iterator
@@ -1132,22 +1112,6 @@ Body_Pass::verify_direct_pass( const WorldModel & world,
             return false;
         }
 
-#if 0
-        int cycle = static_cast< int >
-            ( std::ceil( interception
-                         .getReachCycle( (*it)->pos(),
-                                         ( (*it)->velCount() <= 1 ? &((*it)->vel()) : NULL ),
-                                         ( (*it)->bodyCount() == 0 ? &((*it)->body()) : NULL ),
-                                         (*it)->posCount(),
-                                         1.2, // kickable area + buf
-                                         1.1 ) ) ); // dash speed
-        dlog.addText( Logger::PASS,
-                      "______ opp%d(%.1f %.1f) reach cycle by util = %d.",
-                      (*it)->unum(),
-                      (*it)->pos().x, (*it)->pos().y, cycle );
-#endif
-
-
         Vector2D ball_to_opp = (*it)->pos();
         ball_to_opp -= world.ball().pos();
         ball_to_opp -= first_vel;
@@ -1230,26 +1194,6 @@ Body_Pass::verify_through_pass( const WorldModel & world,
 
     const double receiver_to_target = receiver_pos.dist( target_point );
 
-
-#if 0
-    const Interception interception( world.ball().pos() + first_vel,
-                                     first_speed * ServerParam::i().ballDecay(),
-                                     target_angle );
-    {
-        int cycle = static_cast< int >
-            ( std::ceil( interception.getReachCycle( receiver->pos(),
-                                                     &(receiver->vel()),
-                                                     ( receiver->bodyCount() == 0
-                                                       ? &(receiver->body()) : NULL ),
-                                                     receiver->posCount(),
-                                                     0.9, // kickable area + buf
-                                                     1.0 ) ) ); // dash speed
-        dlog.addText( Logger::PASS,
-                      "______ receiver reach cycle by util = %d.",
-                      cycle );
-    }
-#endif
-
     bool very_aggressive = false;
     if ( target_point.x > 28.0
          && target_point.x > world.ball().pos().x + 20.0 )
@@ -1286,21 +1230,6 @@ Body_Pass::verify_through_pass( const WorldModel & world,
             }
         }
 
-#if 0
-        int cycle = static_cast< int >
-            ( std::ceil( interception
-                         .getReachCycle( (*it)->pos(),
-                                         ( (*it)->velCount() <= 1 ? &((*it)->vel()) : NULL ),
-                                         ( (*it)->bodyCount() == 0 ? &((*it)->body()) : NULL ),
-                                         (*it)->posCount(),
-                                         1.2, // kickable area + buf
-                                         1.1 ) ) ); // dash speed
-        dlog.addText( Logger::PASS,
-                      "______ opp%d(%.1f %.1f) reach cycle by util = %d.",
-                      (*it)->unum(),
-                      (*it)->pos().x, (*it)->pos().y, cycle );
-#endif
-
         const double virtual_dash
             = player_dash_speed * std::min( 2, (*it)->posCount() );
         const double opp_to_target = (*it)->pos().dist( target_point );
@@ -1397,8 +1326,7 @@ Body_Pass::evaluate_routes( const WorldModel & world )
         double x_diff_rate = 1.0;
         {
             double x_diff = it->receive_point_.x - world.self().pos().x;
-            x_diff_rate = std::pow( 0.98,
-                                    std::max( 0.0, 30.0 - x_diff ) );
+            x_diff_rate = std::pow( 0.98, std::max( 0.0, 30.0 - x_diff ) );
         }
         //-----------------------------------------------------------
         double receiver_move_rate = 1.0;
