@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -63,16 +63,14 @@ Body_StopBall::execute( PlayerAgent * agent )
                   << " not ball kickable!"
                   << std::endl;
         dlog.addText( Logger::ACTION,
-                      "%s:%d:  not kickable"
-                      ,__FILE__, __LINE__ );
+                      __FILE__":  not kickable" );
         return false;
     }
 
     if ( ! wm.ball().velValid() )
     {
         dlog.addText( Logger::ACTION,
-                      "%s:%d:  invalid ball velocity. kick to self"
-                      ,__FILE__, __LINE__ );
+                      __FILE__": (execute) invalid ball velocity. kick to self" );
         agent->debugClient().addMessage( "StopBall:InvalidBallVel" );
         // keep current relative position
         Vector2D required_accel = wm.self().vel() - wm.ball().rpos();
