@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -51,13 +51,16 @@
 #include <functional>
 #include <fstream>
 
+#define DEBUG_PROFILE
 //#define DEBUG
+//#define DEBUG_EVALUATE
+//#define DEBUG_THREE_STEP
 
 namespace rcsc {
 
 const double KickTable::NEAR_SIDE_RATE = 0.3;
 const double KickTable::MID_RATE = 0.5;
-const double KickTable::FAR_SIDE_RATE = 0.68;
+const double KickTable::FAR_SIDE_RATE = 0.7;
 
 /*!
  \struct TableCmp
@@ -144,20 +147,21 @@ KickTable::calc_far_dist( const PlayerType & player_type )
     //      0.3 + 0.6*0.7 + 0.085 = 0.865 (=0.985-0.12 -> 0.785)
     // far: 0.3 + 0.7*0.7 + 0.085 = 0.875 (=1.085-0.21)
     //      0.3 + 0.8*0.7 + 0.085 = 0.945 (=1.185-0.24)
-    //return bound( player_type.playerSize() + ServerParam::i().ballSize() + 0.1,
-    //              ( player_type.playerSize()
-    //                + ( player_type.kickableMargin() * FAR_SIDE_RATE )
-    //                + ServerParam::i().ballSize() ),
-    //              player_type.kickableArea() - 0.225 );
 
     //      0.3 + 0.6*0.68 + 0.085 = 0.793 (=0.985-0.192 -> 0.760)
     // far: 0.3 + 0.7*0.68 + 0.085 = 0.861 (=1.085-0.224 -> 0.860)
     //      0.3 + 0.8*0.68 + 0.085 = 0.929 (=1.185-0.256)
+
+    //      0.3 + 0.6*0.675 + 0.085 = 0.79   (=0.985-0.195)
+    // far: 0.3 + 0.7*0.675 + 0.085 = 0.8575 (=1.085-0.2275)
+    //      0.3 + 0.8*0.675 + 0.085 = 0.925  (=1.185-0.26)
+
     return bound( player_type.playerSize() + ServerParam::i().ballSize() + 0.1,
                   ( player_type.playerSize()
                     + ( player_type.kickableMargin() * FAR_SIDE_RATE )
                     + ServerParam::i().ballSize() ),
                   player_type.kickableArea() - 0.2 );
+                  //player_type.kickableArea() - 0.22 );
 }
 
 /*-------------------------------------------------------------------*/
@@ -169,7 +173,7 @@ KickTable::calc_max_velocity( const AngleDeg & target_angle,
                               const double & krate,
                               const Vector2D & ball_vel )
 {
-    const double ball_speed_max2 = std::pow( ServerParam::i().ballSpeedMax(), 2.0 );
+    const double ball_speed_max2 = std::pow( ServerParam::i().ballSpeedMax(), 2 );
     const double max_accel
         = std::min( ServerParam::i().maxPower() * krate,
                     ServerParam::i().ballAccelMax() );
@@ -680,11 +684,17 @@ KickTable::updateState( const WorldModel & world )
     //
     // update current state
     //
+#ifdef DEBUG_PROFILE
     MSecTimer timer;
+#endif
+
     createStateCache( world );
+
+#ifdef DEBUG_PROFILE
     dlog.addText( Logger::KICK,
                   __FILE__": updateState() elapsed %.3f [ms]",
                   timer.elapsedReal() );
+#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -848,7 +858,7 @@ KickTable::checkCollisionAfterRelease( const WorldModel & world,
 
     const PlayerType & self_type = world.self().playerType();
 
-    const double collide_dist2 = std::pow( self_type.playerSize() + ServerParam::i().ballSize(), 2.0 );
+    const double collide_dist2 = std::pow( self_type.playerSize() + ServerParam::i().ballSize(), 2 );
 
     Vector2D self_pos = world.self().pos();
     Vector2D self_vel = world.self().vel();
@@ -937,7 +947,7 @@ KickTable::checkInterfereAt( const WorldModel & world,
                              State & state )
 {
     static const Rect2D penalty_area( Vector2D( ServerParam::i().theirPenaltyAreaLineX(),
-                                                ServerParam::i().penaltyAreaHalfWidth() ),
+                                                - ServerParam::i().penaltyAreaHalfWidth() ),
                                       Size2D( ServerParam::i().penaltyAreaLength(),
                                               ServerParam::i().penaltyAreaWidth() ) );
 
@@ -1299,15 +1309,19 @@ KickTable::simulateOneStep( const WorldModel & world,
 {
     if ( M_current_state.flag_ & SELF_COLLISION )
     {
+#ifdef DEBUG
         dlog.addText( Logger::KICK,
                       "xx__ 1 step: self collision" );
+#endif
         return false;
     }
 
     if ( M_current_state.flag_ & RELEASE_INTERFERE )
     {
+#ifdef DEBUG
         dlog.addText( Logger::KICK,
                       "xx__ 1 step: opponent can interfere after release" );
+#endif
         return false;
     }
 
@@ -1464,8 +1478,8 @@ KickTable::simulateTwoStep( const WorldModel & world,
                 = self_type.kickRand()
                 * ( kick_power / param.maxPower() )
                 * ( current_pos_rate + current_speed_rate );
-            if ( ( my_noise + ball_noise + max_kick_rand ) * 0.9
-                 > my_kickable_area - state.dist_ - 0.1 )
+            if ( ( my_noise + ball_noise + max_kick_rand ) //* 0.9
+                 > my_kickable_area - state.dist_ - 0.05 ) //0.1 )
             {
 #ifdef DEBUG
                 dlog.addText( Logger::KICK,
@@ -1586,9 +1600,11 @@ KickTable::simulateThreeStep( const WorldModel & world,
     int target_angle_index = static_cast< int >( rint( DEST_DIR_DIVS * ( angle_deg / 360.0 ) ) );
     if ( target_angle_index >= DEST_DIR_DIVS ) target_angle_index = 0;
 
+#ifdef DEBUG
     dlog.addText( Logger::KICK,
                   __FILE__": simulateThreeStep() target angle index = %d ",
                   target_angle_index );
+#endif
 
     const std::vector< Path > & table = M_tables[target_angle_index];
 
@@ -1606,7 +1622,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_1st.flag_ & OUT_OF_PITCH )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: skip. out of pitch. state_1st pos=(%.2f %.2f)",
                           state_1st.pos_.x, state_1st.pos_.y );
@@ -1616,7 +1632,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_2nd.flag_ & OUT_OF_PITCH )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: skip. out of pitch. state_2nd pos=(%.2f %.2f)",
                           state_2nd.pos_.x, state_2nd.pos_.y );
@@ -1626,7 +1642,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_1st.flag_ & KICKABLE )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: skip. exist kicable opp. state_1st pos=(%.2f %.2f)",
                           state_1st.pos_.x, state_1st.pos_.y );
@@ -1636,7 +1652,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_2nd.flag_ & KICKABLE )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: skip. exist kicable opp. state_2nd pos=(%.2f %.2f)",
                           state_2nd.pos_.x, state_2nd.pos_.y );
@@ -1646,7 +1662,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_2nd.flag_ & SELF_COLLISION )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: skip. self collision. state_2nd_pos=(%.2f %.2f)",
                           state_2nd.pos_.x, state_2nd.pos_.y );
@@ -1656,7 +1672,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( state_2nd.flag_ & RELEASE_INTERFERE )
         {
-#if 0
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: interfere after release. state_pos=(%.2f %.2f)",
                           state.pos_.x, state.pos_.y );
@@ -1675,7 +1691,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( accel_r2 > current_max_accel2 )
         {
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: failed(1) required_accel=%.3f > max_accel=%.3f",
                           std::sqrt( accel_r2 ), std::sqrt( current_max_accel2 ) );
@@ -1691,12 +1707,12 @@ KickTable::simulateThreeStep( const WorldModel & world,
                 = self_type.kickRand()
                 * ( kick_power / param.maxPower() )
                 * ( current_pos_rate + current_speed_rate );
-            if ( ( my_noise1 + ball_noise + max_kick_rand ) * 0.9
-                 > my_kickable_area - state_1st.dist_ - 0.1 )
+            if ( ( my_noise1 + ball_noise + max_kick_rand ) //* 0.95
+                 > my_kickable_area - state_1st.dist_ - 0.05 ) //0.1 )
             {
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
                 dlog.addText( Logger::KICK,
-                              "xx__ 3 step: failed. buffer is not safety. power=%.3f"
+                              "xx__ 3 step: failed. 1st kick may cause unkickable. power=%.3f"
                               " my_kickable=%.3f state_dist=%.3f,"
                               " noise=%f(my_noise=%f ball_noise=%f kick_rand=%f)",
                               kick_power,
@@ -1717,7 +1733,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
 
         if ( accel_r2 > square( std::min( state_1st.kick_rate_ * max_power, accel_max ) ) )
         {
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: failed(2) required_accel=%.3f > max_accel=%.3f",
                           std::sqrt( accel_r2 ),
@@ -1731,7 +1747,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
         accel_r2 = accel.r2();
         if ( accel_r2 > square( std::min( state_2nd.kick_rate_ * max_power, accel_max ) ) )
         {
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
             dlog.addText( Logger::KICK,
                           "xx__ 3 step: failed(3) required_accel=%.3f > max_accel=%.3f",
                           std::sqrt( accel_r2 ),
@@ -1762,7 +1778,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
                     M_candidates.back().speed_ = std::sqrt( max_speed2 );
                     M_candidates.back().power_ = accel.r() / state_2nd.kick_rate_;
 
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
                     dlog.addText( Logger::KICK,
                                   "____ update max vel (%.2f %.2f) %.3f",
                                   max_vel.x, max_vel.y,
@@ -1784,7 +1800,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
         M_candidates.back().speed_ = first_speed;
         M_candidates.back().power_ = std::sqrt( accel_r2 ) / state_2nd.kick_rate_;
 
-#ifdef DEBUG
+#ifdef DEBUG_THREE_STEP
         dlog.addText( Logger::KICK,
                       "ok__ 3 step: last_power=%.2f sub1=(%.2f %.2f) sub2(%.2f %.2f)",
                       M_candidates.back().power_,
@@ -1794,6 +1810,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
         ++success_count;
     }
 
+#ifdef DEBUG_THREE_STEP
     dlog.addText( Logger::KICK,
                   "simulateThreeKick() solution_size=%d",
                   success_count );
@@ -1803,6 +1820,7 @@ KickTable::simulateThreeStep( const WorldModel & world,
                       "simulateThreeKick() max_speed=%.3f",
                       std::sqrt( max_speed2 ) );
     }
+#endif
 
     return success_count > 0;
 }
@@ -1889,14 +1907,16 @@ KickTable::evaluate( const double & first_speed,
         if ( it->flag_ & KICK_MISS_POSSIBILITY )
         {
             it->score_ -= 30.0;
+#ifdef DEBUG_EVALUATE
             dlog.addText( Logger::KICK,
                           "____ sequence kick miss possibility flag=%x n_kick=%d speed=%.3f last_kick_power=%f",
                           it->flag_,
                           n_kick,
                           it->speed_,
                           it->power_ );
+#endif
         }
-
+#ifdef DEBUG_EVALUATE
         dlog.addText( Logger::KICK,
                       "__ sequence score %.2f flag=%x n_kick=%d speed=%.3f last_kick_power=%f",
                       it->score_,
@@ -1904,6 +1924,7 @@ KickTable::evaluate( const double & first_speed,
                       n_kick,
                       it->speed_,
                       it->power_ );
+#endif
     }
 }
 
@@ -1929,7 +1950,9 @@ KickTable::simulate( const WorldModel & world,
         return false;
     }
 
+#ifdef DEBUG_PROFILE
     MSecTimer timer;
+#endif
 
     double target_speed = bound( 0.0,
                                  first_speed,
@@ -2007,9 +2030,11 @@ KickTable::simulate( const WorldModel & world,
                   sequence.power_,
                   sequence.score_ );
 
+#ifdef DEBUG_PROFILE
     dlog.addText( Logger::KICK,
                   __FILE__": simulate() kick_table_elapsed=%.4f [ms].",
                   timer.elapsedReal() );
+#endif
     return sequence.speed_ >= target_speed - rcsc::EPS;
 }
 
