@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,7 +36,7 @@
 
 namespace rcsc {
 
-class PlayerObject;
+class AbstractPlayerObject;
 
 /*!
   \class Neck_TurnToPlayerOrScan
@@ -46,7 +46,7 @@ class Neck_TurnToPlayerOrScan
     : public NeckAction {
 private:
 
-    const PlayerObject * M_target_player; //!< target player pointer
+    const AbstractPlayerObject * M_target_player; //!< target player pointer
     const int M_count_thr; //!< count threshold value
 
 public:
@@ -56,7 +56,7 @@ public:
       \param count_thr if player's posCount is smaller than this, agent will scan field.
      */
     explicit
-    Neck_TurnToPlayerOrScan( const PlayerObject * target_player,
+    Neck_TurnToPlayerOrScan( const AbstractPlayerObject * target_player,
                              const int count_thr = 1 )
         : M_target_player( target_player )
         , M_count_thr( count_thr )
