@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -38,13 +38,14 @@
 #include "kick_table.h"
 
 #include <rcsc/player/player_agent.h>
-#include <rcsc/player/interception.h>
 #include <rcsc/common/logger.h>
 #include <rcsc/common/server_param.h>
 #include <rcsc/math_util.h>
 #include <rcsc/timer.h>
 
-//#define DEBUG
+// #define DEBUG_PROFILE
+// #define DEBUG_PRINT
+// #define DEBUG_PRINT_LEVEL_2
 
 namespace rcsc {
 
@@ -69,19 +70,26 @@ ShootTable2008::search( const PlayerAgent * agent )
     M_shots.clear();
 
     /////////////////////////////////////////////////////////////////////
-    static const Rect2D
-        shootable_area( Vector2D( ServerParam::i().theirPenaltyAreaLineX() - 5.0, // left
-                                  -ServerParam::i().penaltyAreaHalfWidth() ), // top
-                        Size2D( ServerParam::i().penaltyAreaLength() + 5.0, // length
-                                ServerParam::i().penaltyAreaWidth() ) ); // width
+//     static const Rect2D
+//         shootable_area( Vector2D( ServerParam::i().theirPenaltyAreaLineX() - 5.0, // left
+//                                   -ServerParam::i().penaltyAreaHalfWidth() ), // top
+//                         Size2D( ServerParam::i().penaltyAreaLength() + 5.0, // length
+//                                 ServerParam::i().penaltyAreaWidth() ) ); // width
+    static const Vector2D goal_c( ServerParam::i().pitchHalfLength(), 0.0 );
 
-    if ( ! wm.self().isKickable()
-         || ! shootable_area.contains( wm.ball().pos() ) )
+    if ( ! wm.self().isKickable() )
     {
         return;
     }
 
+    if ( wm.self().pos().dist2( goal_c ) > std::pow( 30.0, 2 ) )
+    {
+        return;
+    }
+
+#ifdef DEBUG_PROFILE
     MSecTimer timer;
+#endif
 
     Vector2D goal_l( ServerParam::i().pitchHalfLength(),
                      -ServerParam::i().goalHalfWidth() );
@@ -100,12 +108,14 @@ ShootTable2008::search( const PlayerAgent * agent )
         goal_r.x = wm.self().pos().x + 1.5;
     }
 
-    const int DIST_DIVS = 15;
+    const int DIST_DIVS = 25;
     const double dist_step = std::fabs( goal_l.y - goal_r.y ) / ( DIST_DIVS - 1 );
 
+#ifdef DEBUG_PRINT
     dlog.addText( Logger::SHOOT,
                   "Shoot search range=(%.1f %.1f)-(%.1f %.1f) dist_step=%.1f",
                   goal_l.x, goal_l.y, goal_r.x, goal_r.y, dist_step );
+#endif
 
     const PlayerObject * goalie = agent->world().getOpponentGoalie();
 
@@ -118,10 +128,12 @@ ShootTable2008::search( const PlayerAgent * agent )
         calculateShotPoint( wm, shot_point, goalie );
     }
 
+#ifdef DEBUG_PROFILE
     dlog.addText( Logger::SHOOT,
                   "Shoot found %d shots. search_shoot_elapsed=%.3f [ms]",
                   (int)M_shots.size(),
                   timer.elapsedReal() );
+#endif
 
 }
 
@@ -182,10 +194,10 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
         Shot shot( shot_point, shot_first_speed, shot_angle );
         shot.score_ = 0;
 
-        if ( canScore( wm, &shot ) )
+        bool one_step = ( shot_first_speed <= max_one_step_speed );
+        if ( canScore( wm, one_step, &shot ) )
         {
             shot.score_ += 100;
-            bool one_step = ( shot_first_speed <= max_one_step_speed );
             if ( one_step )
             {   // one step kick
                 shot.score_ += 100;
@@ -201,10 +213,12 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
             {
                 AngleDeg goalie_angle = ( goalie->pos() - wm.ball().pos() ).th();
                 double angle_diff = ( shot.angle_ - goalie_angle ).abs();
-                goalie_rate = 1.0 - std::exp( - std::pow( angle_diff * 0.1, 2.0 )
-                                              / ( 2.0 * 90.0 * 0.1 ) );
+//                 goalie_rate = 1.0 - std::exp( - std::pow( angle_diff * 0.1, 2 )
+//                                               / ( 2.0 * 90.0 * 0.1 ) );
+                goalie_rate = 1.0 - std::exp( - std::pow( angle_diff * 0.1, 2 )
+                                              / ( 2.0 * 40.0 * 0.1 ) );
                 shot.score_ = static_cast< int >( shot.score_ * goalie_rate );
-#ifdef DEBUG
+#ifdef DEBUG_PRINT
                 dlog.addText( Logger::SHOOT,
                               "--- apply goalie rate. angle_diff=%.1f rate=%.2f",
                               angle_diff, goalie_rate );
@@ -213,6 +227,7 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
 
             shot.score_ = static_cast< int >( shot.score_ * y_rate );
 
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
                           "<<< Shoot score=%d pos(%.1f %.1f)"
                           " angle=%.1f speed=%.1f"
@@ -225,10 +240,11 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
                           y_rate, goalie_rate,
                           ( one_step ? 1 : 0 ),
                           ( shot.goalie_never_reach_ ? 1 : 0 ) );
+#endif
 
             M_shots.push_back( shot );
         }
-#ifdef DEBUG
+#ifdef DEBUG_PRINT
         else
         {
             dlog.addText( Logger::SHOOT,
@@ -249,16 +265,12 @@ ShootTable2008::calculateShotPoint( const WorldModel & wm,
  */
 bool
 ShootTable2008::canScore( const WorldModel & wm,
+                          const bool one_step_kick,
                           Shot * shot )
 {
     static const double opp_x_thr = ServerParam::i().theirPenaltyAreaLineX() - 5.0;
     static const double opp_y_thr = ServerParam::i().penaltyAreaHalfWidth();
-#if 0
-    static const double player_max_speed
-        = ServerParam::i().defaultPlayerSpeedMax();
-    static const double player_control_area
-        = ServerParam::i().defaultKickableArea();
-#endif
+
     // estimate required ball travel step
     const double ball_reach_step
         = calc_length_geom_series( shot->speed_,
@@ -271,19 +283,20 @@ ShootTable2008::canScore( const WorldModel & wm,
         return true;
     }
 
+    const int ball_reach_step_i = static_cast< int >( std::ceil( ball_reach_step ) );
+
+#ifdef DEBUG_PRINT
     dlog.addText( Logger::SHOOT,
-                  "canScore() shot to (%.1f %.1f) vel=(%.2f %.2f)%.3f angle=%.1f",
+                  "canScore() shot to (%.1f %.1f) vel=(%.2f %.2f)%.3f angle=%.1f"
+                  " ball_reach_step=%.3f",
                   shot->point_.x, shot->point_.y,
                   shot->vel_.x, shot->vel_.y,
                   shot->speed_,
-                  shot->angle_.degree() );
-
-    const int ball_reach_step_i = static_cast< int >( std::ceil( ball_reach_step ) );
+                  shot->angle_.degree(),
+                  ball_reach_step );
+#endif
 
     // estimate opponent interception
-    const Interception util( wm.ball().pos() + shot->vel_,
-                             shot->speed_ * ServerParam::i().ballDecay(),
-                             shot->angle_ );
 
     const PlayerPtrCont::const_iterator end = wm.opponentsFromSelf().end();
     for ( PlayerPtrCont::const_iterator it = wm.opponentsFromSelf().begin();
@@ -315,45 +328,36 @@ ShootTable2008::canScore( const WorldModel & wm,
             {
                 continue;
             }
-#if 1
+
+#ifdef DEBUG_PRINT
+            dlog.addText( Logger::SHOOT,
+                          "__ opp %d(%.1f %.1f)",
+                          (*it)->unum(), (*it)->pos().x, (*it)->pos().y );
+#endif
             int cycle = predictOpponentReachStep( wm,
                                                   shot->point_,
                                                   *it,
                                                   wm.ball().pos(),
                                                   shot->vel_,
+                                                  one_step_kick,
                                                   ball_reach_step_i );
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "__ opp %d(%.1f %.1f) reach step=%d, ball_step=%d",
-                          (*it)->unum(),
-                          (*it)->pos().x, (*it)->pos().y,
+                          "____ reach step=%d, ball_step=%d",
                           cycle,
                           ball_reach_step_i );
+#endif
             if ( cycle == 1
                  || cycle < ball_reach_step_i - 1 )
             {
-                return false;
-            }
-#else
-            double cycle = ( std::ceil( util.getReachCycle( (*it)->pos(),
-                                                            &((*it)->vel()),
-                                                            NULL,
-                                                            std::min( 2, (*it)->posCount() ),
-                                                            player_control_area,
-                                                            player_max_speed ) ) );
-
-            if ( cycle < ball_reach_step )
-            {
-#ifdef DEBUG
+#ifdef DEBUG_PRINT
                 dlog.addText( Logger::SHOOT,
-                              "__ opp %d(%.1f %.1f) can reach. reach_step=%.2f < ball_step=%.2f",
+                              "xx__ opp %d(%.1f %.1f) will get the ball",
                               (*it)->unum(),
-                              (*it)->pos().x, (*it)->pos().y,
-                              cycle,
-                              ball_reach_step );
+                              (*it)->pos().x, (*it)->pos().y );
 #endif
                 return false;
             }
-#endif
         }
     }
 
@@ -408,19 +412,23 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
     {
         // estimate the required turn angle
         Vector2D goalie_pos = goalie->inertiaPoint( cycle );
-//             = inertia_n_step_point( goalie->pos(),
-//                                     goalie->vel(),
-//                                     cycle,
-//                                     param.defaultPlayerDecay() );
         Vector2D ball_relative = ball_pos - goalie_pos;
         double ball_dist = ball_relative.r() - seen_dist_noise;
 
         if ( ball_dist < catchable_area )
         {
+#ifdef DEBUG_PRINT
+            dlog.addText( Logger::SHOOT,
+                          "xx__ goalie can catch the ball. cycle=%d ball_pos(%.1f %.1f)"
+                          " ball_dist=%.3f",
+                          cycle,
+                          ball_pos.x, ball_pos.y,
+                          ball_dist );
+#endif
             return true;
         }
 
-        if ( ball_dist < catchable_area + 1.0 )
+        if ( ball_dist < catchable_area + 1.75 ) // 2009-07-02: 1.0 -> 1.75
         {
             shot->goalie_never_reach_ = false;
         }
@@ -442,7 +450,7 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
             = std::max( AngleDeg::asin_deg( catchable_area / ball_dist ),
                         15.0 );
 
-#ifdef DEBUG
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::SHOOT,
                       "__ goalie. cycle=%d ball_pos(%.1f %.1f) g_body=%.0f"
                       " b_angle=%.1f angle_diff=%.1f turn_margin=%.1f",
@@ -467,16 +475,12 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
         }
 
         // simulate dash
-        goalie_pos
-            = inertia_n_step_point( goalie->pos(),
-                                    goalie->vel(),
-                                    n_turn,
-                                    param.defaultPlayerDecay() );
+        goalie_pos = goalie->inertiaPoint( n_turn );
 
         const Vector2D dash_accel = Vector2D::polar2vector( dash_accel_mag,
                                                             ball_angle );
         const int max_dash = ( cycle - 1 - n_turn
-                               + std::min( 5, goalie->posCount() ) );
+                               + bound( 0, goalie->posCount() - 1, 5 ) );
         double goalie_travel = 0.0;
         for ( int i = 0; i < max_dash; ++i )
         {
@@ -486,11 +490,11 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
             goalie_vel *= param.defaultPlayerDecay();
 
             double d = goalie_pos.dist( ball_pos ) - seen_dist_noise;
-#if 0
+#ifdef DEBUG_PRINT_LEVEL_2
             dlog.addText( Logger::SHOOT,
-                          "__ goalie. cycle=%d ball(%.1f %.1f) angle=%.0f"
+                          "__ goalie. cycle=%d turn=%d dash=%d ball(%.1f %.1f) angle=%.0f"
                           " goalie pos(%.1f %.1f) travel=%.1f dist=%.1f turn=%d dash=%d",
-                          cycle,
+                          cycle, n_turn, i + 1,
                           ball_pos.x, ball_pos.y,
                           ball_angle.degree(),
                           goalie_pos.x, goalie_pos.y,
@@ -510,11 +514,13 @@ ShootTable2008::maybeGoalieCatch( const WorldModel & wm,
              - seen_dist_noise
              - catchable_area )
         {
-#ifdef DEBUG
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::SHOOT,
-                          "__ goalie can reach. move_dist=%.3f ball_dist=%.3f",
+                          "xx__ goalie can reach. move_dist=%.3f ball_dist=%.3f"
+                          " (raw_dist=%.3f seen_noise=%.3f catch_area=%.3f",
                           goalie->pos().dist( goalie_pos ) * 1.05,
-                          goalie->pos().dist( ball_pos ) - seen_dist_noise - catchable_area );
+                          goalie->pos().dist( ball_pos ) - seen_dist_noise - catchable_area,
+                          goalie->pos().dist( ball_pos ), seen_dist_noise, catchable_area );
 #endif
             return true;
         }
@@ -538,6 +544,7 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
                                           const PlayerObject * opponent,
                                           const Vector2D & first_ball_pos,
                                           const Vector2D & first_ball_vel,
+                                          const bool one_step_kick,
                                           const int max_step )
 {
     const ServerParam & param = ServerParam::i();
@@ -585,6 +592,12 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
             turn_margin = std::max( turn_margin, 12.0 );
 
             double opp_speed = opponent->vel().r();
+#ifdef DEBUG_PRINT_LEVEL_2
+            dlog.addText( Logger::SHOOT,
+                          "____ speed=%.3f turn_marign=%.1f angle_diff=%.1f",
+                          opp_speed,
+                          turn_margin, angle_diff );
+#endif
             while ( angle_diff > turn_margin )
             {
                 angle_diff -= player_type->effectiveTurn( param.maxMoment(), opp_speed );
@@ -598,15 +611,29 @@ ShootTable2008::predictOpponentReachStep( const WorldModel &,
 
         if ( opp_to_ball_dist < 0.0 )
         {
+#ifdef DEBUG_PRINT_LEVEL_2
+            dlog.addText( Logger::SHOOT,
+                          "xxxx reachable without dash. ball_dist=%.3f",
+                          opp_to_ball_dist + control_area + opponent->distFromSelf() * 0.03 );
+#endif
             return cycle;
         }
 
         int n_step = player_type->cyclesToReachDistance( opp_to_ball_dist );
         n_step += n_turn;
-        n_step -= bound( 0, opponent->posCount() - 1, 2 );
+        //n_step -= bound( 0, opponent->posCount() - 1, 2 );
+        n_step -= bound( 0, opponent->posCount(), 2 );
 
-        if ( n_step <= cycle )
+        if ( n_step < cycle - ( one_step_kick ? 1 : 0 ) )
         {
+#ifdef DEBUG_PRINT_LEVEL_2
+            dlog.addText( Logger::SHOOT,
+                          "xxxx reachable with dash. cycle=%d turn=%d dash=%d n_buf=%d",
+                          cycle,
+                          n_turn,
+                          n_step - n_turn + bound( 0, opponent->posCount() - 1, 2 ),
+                          bound( 0, opponent->posCount() - 1, 2 ) );
+#endif
             return cycle;
         }
 
