@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,19 +35,86 @@
 
 #include "formation.h"
 
+#include "formation_bpn.h"
+#include "formation_cdt.h"
+#include "formation_dt.h"
+#include "formation_knn.h"
+#include "formation_ngnet.h"
+#include "formation_rbf.h"
+#include "formation_sbsp.h"
+#include "formation_static.h"
+#include "formation_uva.h"
+
 #include <sstream>
 
 namespace rcsc {
 
+using namespace formation;
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+Formation::Creators &
+Formation::creators()
+{
+    static Creators s_instance;
+    return s_instance;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+Formation::Ptr
+Formation::create( const std::string & name )
+{
+    Formation::Ptr ptr( static_cast< Formation * >( 0 ) );
+
+    Formation::Creator creator;
+    if ( Formation::creators().getCreator( creator, name ) )
+    {
+        ptr = creator();
+    }
+    else if ( name == FormationBPN::NAME ) ptr = FormationBPN::create();
+    else if ( name == FormationCDT::NAME ) ptr = FormationCDT::create();
+    else if ( name == FormationDT::NAME ) ptr = FormationDT::create();
+    else if ( name == FormationKNN::NAME ) ptr = FormationKNN::create();
+    else if ( name == FormationNGNet::NAME ) ptr = FormationNGNet::create();
+    else if ( name == FormationRBF::NAME ) ptr = FormationRBF::create();
+    else if ( name == FormationSBSP::NAME ) ptr = FormationSBSP::create();
+    else if ( name == FormationStatic::NAME ) ptr = FormationStatic::create();
+    else if ( name == FormationUvA::NAME ) ptr = FormationUvA::create();
+
+    return ptr;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+Formation::Ptr
+Formation::create( std::istream & is )
+{
+    std::string temp, type;
+    is >> temp >> type;
+
+    is.seekg( 0 );
+    return create( type );
+}
+
+
 /*-------------------------------------------------------------------*/
 /*!
 
  */
 Formation::Formation()
+    : M_version( 0 )
+    , M_samples( new SampleDataSet() )
 {
     for ( int i = 0; i < 11; ++i )
     {
-        M_synmetry_number[i] = -1;
+        M_symmetry_number[i] = -1;
     }
 }
 
@@ -67,7 +134,7 @@ Formation::setCenterType( const int unum )
         return;
     }
 
-    M_synmetry_number[unum - 1] = 0;
+    M_symmetry_number[unum - 1] = 0;
 }
 
 /*-------------------------------------------------------------------*/
@@ -85,7 +152,7 @@ Formation::setSideType( const int unum )
         return;
     }
 
-    M_synmetry_number[unum - 1] = -1;
+    M_symmetry_number[unum - 1] = -1;
 }
 
 /*-------------------------------------------------------------------*/
@@ -93,42 +160,43 @@ Formation::setSideType( const int unum )
 
  */
 bool
-Formation::setSynmetryType( const int unum,
-                            const int synmetry_unum )
+Formation::setSymmetryType( const int unum,
+                            const int symmetry_unum,
+                            const std::string & role_name )
 {
     if ( unum < 1 || 11 < unum )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** invalid unum " << unum
+                  << " *** ERROR *** Invalid unum " << unum
                   << std::endl;
         return false;
     }
-    if ( synmetry_unum < 1 || 11 < synmetry_unum )
+    if ( symmetry_unum < 1 || 11 < symmetry_unum )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** invalid synmetry_unum " << unum
+                  << " *** ERROR *** Invalid symmetry unum " << unum
                   << std::endl;
         return false;
     }
-    if ( synmetry_unum == unum )
+    if ( symmetry_unum == unum )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** Never synmetry itself. unum = " << unum
-                  << "  mirro = " << synmetry_unum
+                  << " *** ERROR *** Never symmetry itself. unum=" << unum
+                  << "  symmetry=" << symmetry_unum
                   << std::endl;
         return false;
     }
-    if ( M_synmetry_number[synmetry_unum - 1] > 0 )
+    if ( M_symmetry_number[symmetry_unum - 1] > 0 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** " << synmetry_unum << " is already synmetryed player. "
+                  << " *** ERROR *** " << symmetry_unum << " is already a symmetrical player. "
                   << std::endl;
         return false;
     }
-    if ( M_synmetry_number[synmetry_unum - 1] == 0 )
+    if ( M_symmetry_number[symmetry_unum - 1] == 0 )
     {
         std::cerr << __FILE__ << ":" << __LINE__
-                  << " *** ERROR *** " << synmetry_unum << " is center type player. "
+                  << " *** ERROR *** " << symmetry_unum << " is a center type player. "
                   << std::endl;
         return false;
     }
@@ -138,7 +206,7 @@ Formation::setSynmetryType( const int unum,
     for ( int i = 0; i < 11; ++i )
     {
         if ( i + 1 == unum ) continue;
-        if ( M_synmetry_number[i] == synmetry_unum )
+        if ( M_symmetry_number[i] == symmetry_unum )
         {
             // refered by other player.
             std::cerr << __FILE__ << ":" << __LINE__
@@ -149,9 +217,17 @@ Formation::setSynmetryType( const int unum,
         }
     }
 
-    M_synmetry_number[unum - 1] = synmetry_unum;
+    M_symmetry_number[unum - 1] = symmetry_unum;
+
+    if ( role_name.empty() )
+    {
+        setRoleName( unum, getRoleName( symmetry_unum ) );
+    }
+    else
+    {
+        setRoleName( unum, role_name );
+    }
 
-    setRoleName( unum, getRoleName( synmetry_unum ) );
     return true;
 }
 
@@ -161,52 +237,52 @@ Formation::setSynmetryType( const int unum,
  */
 bool
 Formation::updateRole( const int unum,
-                       const int synmetry_unum,
+                       const int symmetry_unum,
                        const std::string & role_name )
 {
-    if ( getSynmetryNumber( unum ) != synmetry_unum )
+    if ( getSymmetryNumber( unum ) != symmetry_unum )
     {
-        if ( synmetry_unum == 0 )
+        if ( symmetry_unum == 0 )
         {
             createNewRole( unum, role_name, Formation::CENTER );
             return true;
         }
 
-        if ( synmetry_unum < 0 )
+        if ( symmetry_unum < 0 )
         {
             createNewRole( unum, role_name, Formation::SIDE );
             return true;
         }
 
-        // ( synmetry_unum > 0 )
+        // ( symmetry_unum > 0 )
 
-        if ( ! isSideType( synmetry_unum ) )
+        if ( ! isSideType( symmetry_unum ) )
         {
             std::cerr << __FILE__ << ":" << __LINE__
-                      << " You cannot use the player number " << synmetry_unum
-                      << " as a synmetry number."
+                      << " You cannot use the player number " << symmetry_unum
+                      << " as a symmetry number."
                       << std::endl;
             return false;
         }
 
         for ( int i = 1; i <= 11; ++i )
         {
-            if ( i == unum || i == synmetry_unum ) continue;
-            if ( getSynmetryNumber( i ) == synmetry_unum )
+            if ( i == unum || i == symmetry_unum ) continue;
+            if ( getSymmetryNumber( i ) == symmetry_unum )
             {
                 std::cerr << __FILE__ << ":" << __LINE__
-                          << " player number " << synmetry_unum
+                          << " player number " << symmetry_unum
                           << " has already refered by player " << i
                           << std::endl;
                 return false;
             }
         }
 
-        setSynmetryType( unum, synmetry_unum );
+        setSymmetryType( unum, symmetry_unum, role_name );
         return true;
     }
 
-    if ( ! isSynmetryType( unum )
+    if ( ! role_name.empty()
          && getRoleName( unum ) != role_name )
     {
         setRoleName( unum, role_name );
@@ -220,50 +296,32 @@ Formation::updateRole( const int unum,
 /*!
 
  */
-int
-Formation::readName( std::istream & is )
+bool
+Formation::read( std::istream & is )
 {
-    int n_line = 0;
-    std::string line_buf;
+    if ( ! readHeader( is ) ) return false;
+    if ( ! readConf( is ) ) return false;
+    if ( ! readSamples( is ) ) return false;
 
-    while ( std::getline( is, line_buf ) )
+    // check symmetry number circuration reference
+    for ( int i = 0; i < 11; ++i )
     {
-        ++n_line;
-        std::istringstream istr( line_buf );
-
-        std::string tag;
-        istr >> tag;
-        if ( tag.empty()
-             || tag[0] == '#'
-             || ! tag.compare( 0, 2, "//" ) )
-        {
-            continue;
-        }
-
-        if ( tag != "Formation" )
+        int refered_unum = M_symmetry_number[i];
+        if ( refered_unum <= 0 ) continue;
+        if ( M_symmetry_number[refered_unum - 1] > 0 )
         {
             std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** Found invalid tag ["
-                      << tag << "]"
+                      << " *** ERROR *** failed to read formation."
+                      << " Bad symmetrying. player "
+                      << i + 1
+                      << " mirro = " << refered_unum
+                      << " is already symmetrying player"
                       << std::endl;
-            return -1;
+            return false;
         }
-
-        std::string name_str;
-        istr >> name_str;
-        if ( name_str != methodName() )
-        {
-            std::cerr << __FILE__ << ":" << __LINE__
-                      << " *** ERROR *** Invalid formation type name. name must be "
-                      << methodName()
-                      << std::endl;
-            return -1;
-        }
-
-        return n_line;
     }
 
-    return -1;
+    return true;
 }
 
 /*-------------------------------------------------------------------*/
@@ -271,9 +329,124 @@ Formation::readName( std::istream & is )
 
  */
 std::ostream &
-Formation::printName( std::ostream & os ) const
+Formation::print( std::ostream & os ) const
 {
-    os << "Formation " << methodName() << '\n';
+    if ( os ) printHeader( os );
+    if ( os ) printConf( os );
+    if ( os ) printSamples( os );
+
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+Formation::readHeader( std::istream & is )
+{
+    std::string line_buf;
+
+    while ( std::getline( is, line_buf ) )
+    {
+        if ( line_buf.empty()
+             || line_buf[0] == '#'
+             || ! line_buf.compare( 0, 2, "//" ) )
+        {
+            continue;
+        }
+
+        std::istringstream istr( line_buf );
+
+        std::string tag;
+        istr >> tag;
+        if ( tag != "Formation" )
+        {
+            std::cerr << __FILE__ << ":" << __LINE__
+                      << " *** ERROR *** Found invalid tag ["
+                      << tag << "]"
+                      << std::endl;
+            return false;
+        }
+
+        std::string name;
+        istr >> name;
+        if ( name != methodName() )
+        {
+            std::cerr << __FILE__ << ":" << __LINE__
+                      << " *** ERROR *** Unsupported formation type name "
+                      << " [" << name << "]."
+                      << " The name has to be " << methodName()
+                      << std::endl;
+            return false;
+        }
+
+        int ver = 0;
+        if ( istr >> ver )
+        {
+            if ( ver < 0 )
+            {
+                std::cerr << __FILE__ << ":" << __LINE__
+                          << " *** ERROR *** Illegas format version "
+                          << ver
+                          << std::endl;
+                return false;
+            }
+
+            M_version = ver;
+        }
+        else
+        {
+            M_version = 0;
+        }
+
+        return true;
+    }
+
+    return false;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+bool
+Formation::readSamples( std::istream & is )
+{
+    M_samples = SampleDataSet::Ptr( new SampleDataSet() );
+
+    if ( ! M_samples->read( is ) )
+    {
+        M_samples.reset();
+        return false;
+    }
+
+    return true;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+Formation::printHeader( std::ostream & os ) const
+{
+    os << "Formation " << methodName() << ' ' << version() << '\n';
+    return os;
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
+ */
+std::ostream &
+Formation::printSamples( std::ostream & os ) const
+{
+    if ( M_samples )
+    {
+        M_samples->print( os );
+    }
+
     return os;
 }
 
