@@ -1,8 +1,8 @@
 // -*-c++-*-
 
 /*!
-	\file formation_ngnet.h
-	\brief formation data classes using NGNet Header File.
+  \file formation_ngnet.h
+  \brief formation data classes using NGNet Header File.
 */
 
 /*
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,6 +49,8 @@ class FormationNGNet
     : public Formation {
 public:
 
+    static const std::string NAME; //!< type name
+
     /*!
       \brief formation parameter using NGNet
       one Param instance realizes just one player's position.
@@ -101,7 +103,7 @@ public:
         /*!
           \brief get strategic position
           \param ball_pos focus point, usually ball position.
-          \param type side type, if this is synmetry type, the refered param is used.
+          \param type side type, if this is symmetry type, the refered param is used.
           \return coordinate value
         */
         Vector2D getPosition( const Vector2D & ball_pos,
@@ -164,7 +166,7 @@ public:
 
 private:
 
-    //! key: unum. but size is not always 11 if synmetric player exists.
+    //! key: unum. but size is not always 11 if symmetric player exists.
     std::map< int, boost::shared_ptr< Param > > M_param_map;
 
 public:
@@ -182,7 +184,8 @@ public:
     static
     std::string name()
       {
-          return std::string( "NGNet" );
+          //return std::string( "NGNet" );
+          return NAME;
       }
 
     /*!
@@ -190,13 +193,20 @@ public:
       \return pointer to the new object
     */
     static
-    Formation * create()
+    Formation::Ptr create()
       {
-          return ( new FormationNGNet );
+          return Formation::Ptr( new FormationNGNet() );
       }
 
     //--------------------------------------------------------------
 
+    /*!
+      \brief create default formation. assign role and initial positions.
+      \return snapshow variable for the initial stat(ball pos=(0,0)).
+    */
+    virtual
+    void createDefaultData();
+
     /*!
       \brief get the name of this formation
       \return name string
@@ -207,13 +217,6 @@ public:
           return FormationNGNet::name();
       }
 
-    /*!
-      \brief create default formation. assign role and initial positions.
-      \return snapshow variable for the initial stat(ball pos=(0,0)).
-    */
-    virtual
-    Snapshot createDefaultParam();
-
 protected:
 
     /*!
@@ -267,25 +270,27 @@ public:
 
     /*!
       \brief update formation paramter using training data set
-      \param train_data training data container
     */
     virtual
-    void train( const std::list< Snapshot > & train_data );
+    void train();
+
+protected:
 
     /*!
-      \brief restore data from the input stream.
+      \brief restore conf data from the input stream.
       \param is reference to the input stream.
-      \return parsing result
+      \return pasing result.
     */
-    bool read( std::istream & is );
+    virtual
+    bool readConf( std::istream & is );
 
     /*!
       \brief put data to the output stream.
       \param os reference to the output stream
       \return reference to the output stream
     */
-    std::ostream & print( std::ostream & os ) const;
-
+    virtual
+    std::ostream & printConf( std::ostream & os ) const;
 
 private:
 
