@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  this library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -149,22 +149,22 @@ DelaunayTriangulation::clear()
 
     M_edge_count = M_tri_count = 0;
 
-    for ( std::map< int, TrianglePtr >::iterator it = M_triangle_map.begin();
-          it != M_triangle_map.end();
+    for ( TriangleCont::iterator it = M_triangles.begin();
+          it != M_triangles.end();
           ++it )
     {
         delete it->second;
     }
 
-    for ( std::map< int, EdgePtr >::iterator it = M_edge_map.begin();
-          it != M_edge_map.end();
+    for ( EdgeCont::iterator it = M_edges.begin();
+          it != M_edges.end();
           ++it )
     {
         delete it->second;
     }
 
-    M_triangle_map.clear();
-    M_edge_map.clear();
+    M_triangles.clear();
+    M_edges.clear();
     M_vertices.clear();
 
     //std::cout << "clear() end" << std::endl;
@@ -173,6 +173,26 @@ DelaunayTriangulation::clear()
 /*-------------------------------------------------------------------*/
 /*!
 
+*/
+void
+DelaunayTriangulation::addVertices( const std::vector< Vector2D > & v )
+{
+    M_vertices.reserve( M_vertices.size() + v.size() );
+
+    int id = M_vertices.size();
+
+    const std::vector< Vector2D >::const_iterator end = v.end();
+    for ( std::vector< Vector2D >::const_iterator it = v.begin();
+          it != end;
+          ++it, ++id )
+    {
+        M_vertices.push_back( Vertex( id, it->x, it->y ) );
+    }
+}
+
+/*-------------------------------------------------------------------*/
+/*!
+
 */
 void
 DelaunayTriangulation::createInitialTriangle( const Rect2D & region )
@@ -232,7 +252,7 @@ DelaunayTriangulation::createInitialTriangle()
         return;
     }
 
-    std::vector< Vertex >::iterator vit = M_vertices.begin();
+    VertexCont::iterator vit = M_vertices.begin();
 
     double min_x = vit->pos().x;
     double max_x = vit->pos().x;
@@ -240,7 +260,7 @@ DelaunayTriangulation::createInitialTriangle()
     double max_y = vit->pos().y;
 
     ++vit;
-    const std::vector< Vertex >::iterator vend = M_vertices.end();
+    const VertexCont::iterator vend = M_vertices.end();
     for ( ; vit != vend; ++vit )
     {
         if ( vit->pos().x < min_x ) min_x = vit->pos().x;
@@ -263,8 +283,8 @@ DelaunayTriangulation::removeInitialVertices()
     std::vector< EdgePtr > removed_edges;
 
     // search removed edges that has initial vertex
-    const std::map< int, EdgePtr >::iterator map_end = M_edge_map.end();
-    for ( std::map< int, EdgePtr >::iterator it = M_edge_map.begin();
+    const EdgeCont::iterator map_end = M_edges.end();
+    for ( EdgeCont::iterator it = M_edges.begin();
           it != map_end;
           ++it )
     {
@@ -333,8 +353,8 @@ DelaunayTriangulation::findNearestVertex( const Vector2D & pos ) const
     const Vertex * candidate = static_cast< Vertex * >( 0 );
 
     double min_dist2 = 10000000.0;
-    const std::vector< Vertex >::const_iterator end = M_vertices.end();
-    for ( std::vector< Vertex >::const_iterator it = M_vertices.begin();
+    const VertexCont::const_iterator end = M_vertices.end();
+    for ( VertexCont::const_iterator it = M_vertices.begin();
           it != end;
           ++it )
     {
@@ -364,8 +384,8 @@ DelaunayTriangulation::compute()
         return;
     }
 
-    if ( M_triangle_map.empty()
-         || M_triangle_map.size() > 3 )
+    if ( M_triangles.empty()
+         || M_triangles.size() > 3 )
     {
         //std::cout << "compute() create initial triangle no arg" << std::endl;
         createInitialTriangle();
@@ -373,8 +393,8 @@ DelaunayTriangulation::compute()
 
     //std::cout << "********** compute() start **********" << std::endl;
     int loop = 0;
-    const std::vector< Vertex >::iterator vend = M_vertices.end();
-    for ( std::vector< Vertex >::iterator vit = M_vertices.begin();
+    const VertexCont::iterator vend = M_vertices.end();
+    for ( VertexCont::iterator vit = M_vertices.begin();
           vit != vend;
           ++vit )
     {
@@ -416,11 +436,11 @@ DelaunayTriangulation::compute()
 #ifdef DEBUG
         std::cout << __FILE__ << ':' << __LINE__
                   << " ----- result of loop " << loop
-                  << " edge num= " << M_edge_map.size()
-                  << " triangle num= " << M_triangle_map.size()
+                  << " edge num= " << M_edges.size()
+                  << " triangle num= " << M_triangles.size()
                   << std::endl;
-        for ( std::map< int, TrianglePtr >::iterator it = M_triangle_map.begin();
-              it != M_triangle_map.end();
+        for ( TriangleCont::iterator it = M_triangles.begin();
+              it != M_triangles.end();
               ++it )
         {
             std::cout << "  triangle " << it->second->id()
@@ -438,12 +458,12 @@ DelaunayTriangulation::compute()
     std::cout << __FILE__ << ':' << __LINE__
               << " compute() end\n"
               << "----- result of trianglation "
-              << " edge num= " << M_edge_map.size()
-              << " triangle num= " << M_triangle_map.size()
+              << " edge num= " << M_edges<.size()
+              << " triangle num= " << M_triangles.size()
               << std::endl;
 
-    for ( std::map< int, TrianglePtr >::iterator it = M_triangle_map.begin();
-          it != M_triangle_map.end();
+    for ( TriangleCont::iterator it = M_triangles.begin();
+          it != M_triangles.end();
           ++it )
     {
         std::cout << "  triangle " << it->second->id()
@@ -916,8 +936,8 @@ DelaunayTriangulation::ContainedType
 DelaunayTriangulation::findTriangleContains( const Vector2D & pos,
                                              TrianglePtr * sol ) const
 {
-    const std::map< int, TrianglePtr >::const_iterator end = M_triangle_map.end();
-    for ( std::map< int, TrianglePtr >::const_iterator it = M_triangle_map.begin();
+    const TriangleCont::const_iterator end = M_triangles.end();
+    for ( TriangleCont::const_iterator it = M_triangles.begin();
           it != end;
           ++it )
     {
