@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -169,6 +169,11 @@ public:
     */
     std::ostream & makeCommand( std::ostream & to );
 
+    /*!
+      \brief delete all command objects and say messages.
+     */
+    void clearAllCommands();
+
     ///////////////////////////////////////////////////////////////
     /*!
       \brief get const pointer to the player's body command object
