@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,18 +35,14 @@
 
 #include "audio_sensor.h"
 
-#include "audio_codec.h"
-#include "logger.h"
 #include "freeform_parser.h"
-#include "say_message_parser.h"
 
-#include <rcsc/common/server_param.h>
+#include <rcsc/common/say_message_parser.h>
+#include <rcsc/common/logger.h>
 #include <rcsc/math_util.h>
 
 #include <cstdio>
-#include <cstdlib>
 #include <cstring>
-#include <cmath>
 
 namespace rcsc {
 
@@ -79,7 +75,8 @@ AudioSensor::addParser( boost::shared_ptr< SayMessageParser > parser )
         return;
     }
 
-    if ( M_say_message_parsers.find( parser->header() ) != M_say_message_parsers.end() )
+    if ( M_say_message_parsers.find( parser->header() )
+         != M_say_message_parsers.end() )
     {
         std::cerr << __FILE__ << ":" << __LINE__
                   << " ***ERROR*** AudioSensor::addSayMessageParser()"
@@ -89,27 +86,8 @@ AudioSensor::addParser( boost::shared_ptr< SayMessageParser > parser )
         return;
     }
 
-    M_say_message_parsers.insert( std::pair< char, boost::shared_ptr< SayMessageParser > >
-                                  ( parser->header(), parser ) );
-
-#if 0
-    for ( std::vector< boost::shared_ptr< SayMessageParser > >::iterator
-              it = M_say_message_parsers.begin();
-          it != M_say_message_parsers.end();
-          ++it )
-    {
-        if ( (*it)->header() == parser->header() )
-        {
-            std::cerr << "***ERROR*** AudioSensor::addSayMessageParser()"
-                      << " header [" << parser->header()
-                      << "] has already been used."
-                  << std::endl;
-            return;
-        }
-    }
-
-    M_say_message_parsers.push_back( parser );
-#endif
+    M_say_message_parsers.insert( ParserMap::value_type( parser->header(),
+                                                         parser ) );
 }
 
 /*-------------------------------------------------------------------*/
@@ -117,15 +95,12 @@ AudioSensor::addParser( boost::shared_ptr< SayMessageParser > parser )
 
  */
 void
-AudioSensor::removeSayMessageParser( const char header )
+AudioSensor::removeParser( const char header )
 {
-    //M_say_message_parsers.erase( header );
-
-    std::map< char, boost::shared_ptr< SayMessageParser > >::iterator
-        it = M_say_message_parsers.find( header );
+    ParserMap::iterator it = M_say_message_parsers.find( header );
     if ( it == M_say_message_parsers.end() )
     {
-        std::cerr << "***WARNING*** AudioSensor::removeSayMessageParser()"
+        std::cerr << "***WARNING*** AudioSensor::removeParser()"
                   << " header [" << header
                   << "] is not registered."
                   << std::endl;
@@ -133,23 +108,6 @@ AudioSensor::removeSayMessageParser( const char header )
     }
 
     M_say_message_parsers.erase( it );
-
-#if 0
-    std::vector< boost::shared_ptr< SayMessageParser > >::iterator
-        it = M_say_message_parsers.begin();
-
-    while ( it != M_say_message_parsers.end() )
-    {
-        if ( (*it)->header() == header )
-        {
-            it = M_say_message_parsers.erase( it );
-        }
-        else
-        {
-            ++it;
-        }
-    }
-#endif
 }
 
 /*-------------------------------------------------------------------*/
@@ -157,7 +115,7 @@ AudioSensor::removeSayMessageParser( const char header )
 
  */
 void
-AudioSensor::addParser( boost::shared_ptr< FreeformParser > parser )
+AudioSensor::setFreeformParser( boost::shared_ptr< FreeformParser > parser )
 {
     M_freeform_parser = parser;
 }
@@ -196,38 +154,44 @@ AudioSensor::parsePlayerMessage( const char * msg,
                   << std::endl;
         return;
     }
-
     msg += n_read;
 
-    while ( *msg != '\0' && *msg == ' ' ) ++msg;
-    if ( *msg == '\"' ) ++msg;
+    while ( *msg == ' ' ) ++msg;
 
-    std::size_t len = 0;
-    while ( *(msg + len) != '\0'
-            && *(msg + len) != '\"' )
+    char end_char = ')';
+    if ( *msg == '\"' )
     {
-        ++len;
-    }
-
-    if ( len == 0 )
-    {
-        std::cerr << "***ERROR*** AudioSensor::parsePlayer."
-                  << " Empty message "
-                  << std::endl;
-        dlog.addText( Logger::SENSOR,
-                      "AudioSensor: Empty message" );
-        return;
+        end_char = '\"';
+        ++msg;
     }
 
     HearMessage message;
     message.unum_ = unum;
     message.dir_ = dir;
-    message.str_.assign( msg, len );
+    message.str_ = msg;
+
+    std::string::size_type pos = message.str_.rfind( end_char );
+    if ( pos == std::string::npos )
+    {
+        std::cerr << "***ERROR*** AudioSensor::parsePlayerMessage."
+                  << " Illegal message. [" << msg << ']'
+                  << std::endl;
+        return;
+    }
+    message.str_.erase( pos );
+
+    if ( message.str_.empty() )
+    {
+        // empty message
+        return;
+    }
 
     if ( ! std::strncmp( sender, "our", 3 ) )
     {
         if ( M_teammate_message_time != current )
         {
+            dlog.addText( Logger::SENSOR,
+                          __FILE__" (parsePlayerMessage) clear old data" );
             M_teammate_message_time = current;
             M_teammate_messages.clear();
         }
@@ -293,8 +257,8 @@ AudioSensor::parseCoachMessage( const char * msg,
 
     // clang message
 
-    int n_read = 0;
     char msg_type[32];
+    int n_read = 0;
 
     if ( std::sscanf( msg, "(%31[^ ] %n ",
                       msg_type, &n_read ) != 1 )
@@ -304,7 +268,6 @@ AudioSensor::parseCoachMessage( const char * msg,
                   << std::endl;
         return;
     }
-
     msg += n_read;
 
     if ( std::strcmp( msg_type, "freeform" ) != 0 )
@@ -317,39 +280,46 @@ AudioSensor::parseCoachMessage( const char * msg,
         return;
     }
 
-    bool quoted = false;
-    if ( *msg != '\0'
-         && *msg == '\"' )
+    while ( *msg == ' ' ) ++msg;
+
+    bool quated = false;
+    if ( *msg == '\"' )
     {
+        quated = true;
         ++msg;
-        quoted = true;
     }
 
     M_freeform_message_time = current;
-    M_freeform_message.assign( msg );
+    M_freeform_message = msg;
 
-    // remove quotation
-    if ( quoted )
+    // remove quotation or parenthesis
+    if ( quated )
     {
-        std::string::size_type qpos = M_freeform_message.find_last_of( '\"' );
-        if ( qpos != std::string::npos )
+        std::string::size_type pos = M_freeform_message.find_last_of( '\"' );
+        if ( pos == std::string::npos )
         {
-            M_freeform_message.erase( qpos );
+            std::cerr << "***ERROR*** AudioSensor::parseCoachMessage."
+                  << " Illegal quated message. [" << msg << ']'
+                      << std::endl;
+            return;
         }
+        M_freeform_message.erase( pos );
     }
     else
     {
-        // remove last two paren
-        for ( int i = 0; i < 2; ++i )
+        std::string::size_type pos = M_freeform_message.find_last_not_of( ')' );
+        if ( pos == std::string::npos
+             || pos == M_freeform_message.length() - 1 )
         {
-            std::string::size_type ppos = M_freeform_message.find_last_of( ')' );
-            if ( ppos != std::string::npos )
-            {
-                M_freeform_message.erase( ppos );
-            }
+            std::cerr << "***ERROR*** AudioSensor::parseCoachMessage."
+                  << " Illegal quated message. [" << msg << ']'
+                      << std::endl;
+            return;
         }
+        M_freeform_message.erase( pos + 1 );
     }
 
+
     //std::cerr << current << ": "
     //          << "recv freeform message. ["
     //          << M_freeform_message.str_ << ']'
@@ -372,8 +342,8 @@ AudioSensor::parseTrainerMessage( const char * msg,
 {
 
     // (hear <time> referee <msg>) : v7-
-    // (hear <time> coach "<msg>")   : v7+
-    // (hear <time> coach <clang>)   : v7+
+    // (hear <time> coach "<msg>") : v7+
+    // (hear <time> coach <clang>) : v7+
 
     int n_read = 0;
     long cycle;
@@ -387,38 +357,32 @@ AudioSensor::parseTrainerMessage( const char * msg,
                   << std::endl;
         return;
     }
-
     msg += n_read;
 
-    bool quoted = false;
-    if ( *msg != '\0'
-         && *msg == '\"' )
+    while ( *msg == ' ' ) ++msg;
+
+    char end_char = ')';
+    if ( *msg == '\"' )
     {
+        end_char = '\"';
         ++msg;
-        quoted = true;
     }
 
     M_trainer_message_time = current;
     M_trainer_message.erase();
-    M_trainer_message.assign( msg );
+    M_trainer_message = msg;
 
-    // remove quotation
-    if ( quoted )
+    // remove quotation or parenthesis
+    std::string::size_type pos = M_trainer_message.rfind( end_char );
+    if ( pos == std::string::npos )
     {
-        std::string::size_type qpos = M_trainer_message.find_last_of( '\"' );
-        if ( qpos != std::string::npos )
-        {
-            M_trainer_message.erase( qpos );
-        }
+        std::cerr << "***ERROR*** CoachAudioSensor::parsePlayerMessage."
+                  << " Illegal quated message. [" << msg << ']'
+                  << std::endl;
         return;
     }
 
-    // remove last paren
-    std::string::size_type ppos = M_trainer_message.find_last_of( ')' );
-    if ( ppos != std::string::npos )
-    {
-        M_trainer_message.erase( ppos );
-    }
+    M_trainer_message.erase( pos );
 }
 
 /*-------------------------------------------------------------------*/
@@ -433,22 +397,22 @@ AudioSensor::parseTeammateMessage( const HearMessage & message )
         return;
     }
 
-    const std::map< char, boost::shared_ptr< SayMessageParser > >::iterator
-        end = M_say_message_parsers.end();
+    const ParserMap::iterator end = M_say_message_parsers.end();
 
     const char * msg = message.str_.c_str();
 
     while ( *msg != '\0' )
     {
+        const char tag = *msg;
+
         int len = 0;
 
-        std::map< char, boost::shared_ptr< SayMessageParser > >::iterator
-            it = M_say_message_parsers.find( *msg );
+        ParserMap::iterator it = M_say_message_parsers.find( tag );
 
         if ( it == end )
         {
             dlog.addText( Logger::SENSOR,
-                          "AudioSensor: unsupported message [%s] in [%s]",
+                          __FILE__" (parseTeammateMessage) unsupported message [%s] in [%s]",
                           msg, message.str_.c_str() );
             return;
         }
