@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,6 +45,8 @@
 
 #include <iostream>
 
+// #define DEBUG_PRINT
+
 namespace rcsc {
 
 int BallObject::S_pos_count_thr = 10;
@@ -77,7 +79,7 @@ BallObject::State::State()
   , heard_vel_( 0.0, 0.0 )
   , heard_vel_count_( 1000 )
   , lost_count_( 0 )
-  , ghost_( false )
+  , ghost_count_( 0 )
 {
 
 }
@@ -117,12 +119,19 @@ BallObject::set_count_thr( const int pos_thr,
 void
 BallObject::setGhost( const GameTime & )
 {
-    M_state.pos_count_ = 1000;
-    M_state.rpos_count_ = 1000;
-    M_state.lost_count_ = 0;
-    M_state.ghost_ = true;
+    if ( M_state.ghost_count_ > 0 )
+    {
+        M_state.pos_count_ = 1000;
+        M_state.rpos_count_ = 1000;
+        M_state.lost_count_ = 0;
+        M_state.ghost_count_ += 1;
 
-    M_dist_from_self = 1000.0;
+        M_dist_from_self = 1000.0;
+    }
+    else
+    {
+        M_state.ghost_count_ = 1;
+    }
 }
 
 /*-------------------------------------------------------------------*/
@@ -162,10 +171,12 @@ BallObject::update( const ActionEffector & act,
 
             new_vel += accel;
 
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          "Ball update. queued_kick_accel=(%.3f, %.3f) new_vel=(%.3f %.3f)",
+                          __FILE__" (update) queued_kick_accel=(%.3f, %.3f) new_vel=(%.3f %.3f)",
                           accel.x, accel.y,
                           new_vel.x, new_vel.y );
+#endif
         }
 
         // check max vel
@@ -204,13 +215,17 @@ BallObject::update( const ActionEffector & act,
     {
         // if setplay playmode, ball does not move until playmode change to playon.
 
-        if ( pmode == GameMode::BeforeKickOff )
+        if ( pmode == GameMode::BeforeKickOff
+             || pmode == GameMode::KickOff_ )
         {
             M_state.pos_.assign( 0.0, 0.0 );
             M_state.pos_count_ = 0;
             M_state.seen_pos_.assign( 0.0, 0.0 );
+
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          "Ball update. before_kick_off. set to center." );
+                          __FILE__" (update) before_kick_off. set to center." );
+#endif
         }
         // if I didin't see the ball in this setplay playmode,
         // we must check the ball first.
@@ -221,18 +236,22 @@ BallObject::update( const ActionEffector & act,
         {
             // NOT seen at last cycle, but internal info means ball visible.
             // !!! IMPORTANT to check the ghost
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          "Ball update. SetPlay. but not seen. posCount=%d"
+                          __FILE__" (update) SetPlay. but not seen. posCount=%d"
                           " rposCount=%d. distFromSelf=%.3f",
                           posCount(), rposCount(), distFromSelf() );
+#endif
             M_state.pos_count_ = 1000;
         }
         else
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          "Ball update. SetPlay. seen once. posCount=%d"
+                          __FILE__" (update) SetPlay. seen once. posCount=%d"
                           " rposCount=%d distFromSelf=%.3f",
                           posCount(), rposCount(), distFromSelf() );
+#endif
             M_state.pos_count_ = 1;
         }
 
@@ -266,7 +285,7 @@ BallObject::update( const ActionEffector & act,
     M_state.heard_vel_count_ = std::min( 1000, M_state.heard_vel_count_ + 1 );
     M_state.lost_count_ = std::min( 1000, M_state.lost_count_ + 1 );
 
-    M_state.ghost_ = false;
+    // M_state.ghost_count_ = 0;
 
     // set previous rpos
     M_rpos_prev = rpos();
@@ -291,6 +310,10 @@ BallObject::updateByFullstate( const Vector2D & pos,
     M_state.rpos_error_.assign( 0.0, 0.0 );
     M_state.rpos_count_ = 0;
 
+    M_state.seen_pos_ = pos;
+    M_state.seen_rpos_ = M_state.rpos_;
+    M_state.seen_pos_count_ = 0;
+
     M_state.vel_ = vel;
     M_state.vel_error_.assign( 0.0, 0.0 );
     M_state.vel_count_ = 0;
@@ -299,6 +322,8 @@ BallObject::updateByFullstate( const Vector2D & pos,
     M_state.seen_vel_count_ = 0;
 
     M_state.lost_count_ = 0;
+
+    M_state.ghost_count_ = 0;
 }
 
 /*-------------------------------------------------------------------*/
@@ -425,6 +450,7 @@ BallObject::updatePos( const Vector2D & pos,
     M_state.seen_pos_count_ = 0;
     M_state.seen_rpos_ = rpos;
     M_state.lost_count_ = 0;
+    M_state.ghost_count_ = 0;
 }
 
 /*-------------------------------------------------------------------*/
@@ -451,6 +477,7 @@ BallObject::updateAll( const Vector2D & pos,
 */
 void
 BallObject::updateByHear( const ActionEffector & act,
+                          const double & sender_to_ball_dist,
                           const Vector2D & heard_pos,
                           const Vector2D & heard_vel,
                           const GameTime & )
@@ -462,39 +489,89 @@ BallObject::updateByHear( const ActionEffector & act,
 
     if ( act.lastBodyCommandType() == PlayerCommand::KICK )
     {
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": Ball. updateByHeardInfo. last command is kick."
+                      __FILE__" (updateByHear) last command is kick."
                       " heard_pos=(%.2f, %.2f)"
                       " heard_vel=(%.2f, %.2f)",
                       heard_pos.x, heard_pos.y,
                       heard_vel.x, heard_vel.y );
+#endif
         return;
     }
 
-    if ( M_state.ghost_ )
+    if ( M_state.ghost_count_ > 0 )
     {
         if ( heard_pos.dist( pos() ) < 3.0 )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": Ball. updateByHeardInfo. ghost detected. heard_pos=(%.2f, %.2f)"
+                          __FILE__" (updateByHear) ghost detected. heard_pos=(%.2f, %.2f)"
                           " heard_vel=(%.2f, %.2f)",
                           heard_pos.x, heard_pos.y,
                           heard_vel.x, heard_vel.y );
+#endif
+            M_state.pos_ = heard_pos;
+            M_state.pos_count_ = 1;
+            M_state.vel_ = heard_vel;
+            M_state.vel_count_ = 1;
             return;
         }
     }
 
+    if ( posCount() >= 1
+         && ( heard_pos.dist2( pos() ) > std::pow( 2.0, 2 )
+              || std::fabs( heard_vel.x - vel().x ) > 1.0
+              || std::fabs( heard_vel.y - vel().y ) > 1.0 )
+         )
+    {
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::WORLD,
+                      __FILE__" (updateByHear) big difference from last internal state."
+                      " heard_pos=(%.2f, %.2f)"
+                      " heard_vel=(%.2f, %.2f)",
+                      heard_pos.x, heard_pos.y,
+                      heard_vel.x, heard_vel.y );
+#endif
+        M_state.pos_ = heard_pos;
+        M_state.pos_count_ = 1;
+        M_state.vel_ = heard_vel;
+        M_state.vel_count_ = 1;
+        return;
+    }
+
     const double prev_dist = ( M_rpos_prev.valid()
                                ? M_rpos_prev.r()
                                : 1000.0 );
 
-    if ( posCount() >= 2 )
+    if ( sender_to_ball_dist < 5.0
+         && posCount() >= 1 )
     {
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": Ball. updateByHeardInfo. heard_pos=(%.2f, %.2f)"
+                      __FILE__" (updateByHear) sent from the player near to the ball."
+                      " heard_pos=(%.2f, %.2f)"
                       " heard_vel=(%.2f, %.2f)",
                       heard_pos.x, heard_pos.y,
                       heard_vel.x, heard_vel.y );
+#endif
+        M_state.pos_ = heard_pos;
+        M_state.pos_count_ = 1;
+        M_state.vel_ = heard_vel;
+        M_state.vel_count_ = 1;
+    }
+    else if ( //sender_to_ball_dist < prev_dist
+             sender_to_ball_dist < 20.0
+              && posCount() >= 2 )
+    {
+#ifdef DEBUG_PRINT
+        dlog.addText( Logger::WORLD,
+                      __FILE__" (updateByHear) low accuracy."
+                      " heard_pos=(%.2f, %.2f)"
+                      " heard_vel=(%.2f, %.2f)",
+                      heard_pos.x, heard_pos.y,
+                      heard_vel.x, heard_vel.y );
+#endif
         M_state.pos_ = heard_pos;
         M_state.pos_count_ = 1;
         M_state.vel_ = heard_vel;
@@ -503,10 +580,16 @@ BallObject::updateByHear( const ActionEffector & act,
     else if ( ( prev_dist > 1.4 && velCount() >= 3 )
               || velCount() >= 4 )
     {
+#ifdef DEBUG_PRINT
         dlog.addText( Logger::WORLD,
-                      __FILE__": Ball. updateByHeardInfo. vel only"
+                      __FILE__" (updateByHear) low vel accuracy."
+                      " sender_ball_dist=%.3f prev_my_dist=%.3f"
                       " heard_vel=(%.2f, %.2f)",
+                      sender_to_ball_dist, prev_dist,
                       heard_vel.x, heard_vel.y );
+#endif
+        M_state.pos_ = heard_pos;
+        M_state.pos_count_ = 1;
         M_state.vel_ = heard_vel;
         M_state.vel_count_ = 1;
     }
@@ -533,10 +616,11 @@ BallObject::updateSelfRelated( const SelfObject & self )
         if ( M_rpos_prev.valid()
              && self.lastMove().valid() )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": Ball. updateSelfRelated. update rpos using self move(%.2f %.2f)",
+                          __FILE__" (updateSelfRelated) update rpos using self move(%.2f %.2f)",
                           self.lastMove().x, self.lastMove().y );
-
+#endif
             // M_rpos_prev is updated in update()
             M_state.rpos_
                 = M_rpos_prev
@@ -553,10 +637,11 @@ BallObject::updateSelfRelated( const SelfObject & self )
         if ( rpos().valid()
              && posCount() > rposCount() )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": Ball. updateSelfRelated. set pos by rpos(%.2f %.2f)",
+                          __FILE__" (updateSelfRelated) set pos by rpos(%.2f %.2f)",
                           rpos().x, rpos().y );
-
+#endif
             M_state.pos_ = self.pos() + this->rpos();
             M_state.pos_error_ = self.posError() + this->rposError();
             M_dist_from_self = rpos().r();
@@ -564,9 +649,10 @@ BallObject::updateSelfRelated( const SelfObject & self )
         }
         else if ( posValid() )
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": Ball. updateSelfRelated. set rpos by pos" );
-
+                          __FILE__" (updateSelfRelated) set rpos by pos" );
+#endif
             M_state.rpos_ = pos() - self.pos();
             M_state.rpos_error_ = posError() + self.posError();
             M_dist_from_self = rpos().r();
@@ -574,9 +660,10 @@ BallObject::updateSelfRelated( const SelfObject & self )
         }
         else
         {
+#ifdef DEBUG_PRINT
             dlog.addText( Logger::WORLD,
-                          __FILE__": Ball. updateSelfRelated. failed" );
-
+                          __FILE__" (updateSelfRelated) failed" );
+#endif
             M_dist_from_self = 1000.0;
             M_angle_from_self = 0.0;
         }
