@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,7 +34,8 @@
 #endif
 
 #include "intercept_table.h"
-#include "self_intercept.h"
+//#include "self_intercept.h"
+#include "self_intercept_v13.h"
 #include "player_intercept.h"
 #include "world_model.h"
 #include "player_object.h"
@@ -45,6 +46,8 @@
 
 #include <algorithm>
 
+// #define DEBUG_PRINT
+
 namespace rcsc {
 
 const std::size_t InterceptTable::MAX_CYCLE = 30;
@@ -81,12 +84,12 @@ void
 InterceptTable::clear()
 {
     M_ball_pos_cache.clear();
-    //M_ball_vel_cache.clear();
 
     M_self_reach_cycle = 1000;
     M_self_exhaust_reach_cycle = 1000;
     M_teammate_reach_cycle = 1000;
     M_second_teammate_reach_cycle = 1000;
+    M_goalie_reach_cycle = 1000;
     M_opponent_reach_cycle = 1000;
     M_second_opponent_reach_cycle = 1000;
 
@@ -114,7 +117,7 @@ InterceptTable::update()
     s_update_time = M_world.time();
 
     dlog.addText( Logger::INTERCEPT,
-                  "InterceptTable update" );
+                  __FILE__" (update)" );
 
     // clear all data
     this->clear();
@@ -133,17 +136,19 @@ InterceptTable::update()
          || ! M_world.ball().posValid() )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "InterceptTable Invalid self or ball pos" );
+                      __FILE__" (update) Invalid self or ball pos" );
         return;
     }
 
+#ifdef DEBUG
     if ( M_world.self().isKickable()
          || M_world.existKickableTeammate()
          || M_world.existKickableOpponent() )
     {
         dlog.addText( Logger::INTERCEPT,
-                      "InterceptTable Already exist kickable player" );
+                      __FILE__" (update) Already exist kickable player" );
     }
+#endif
 
     createBallCache();
 
@@ -337,7 +342,6 @@ InterceptTable::createBallCache()
     Vector2D bvel = M_world.ball().vel();
 
     M_ball_pos_cache.push_back( bpos );
-    //M_ball_vel_cache.push_back( bvel );
 
     if ( M_world.self().isKickable() )
     {
@@ -348,25 +352,27 @@ InterceptTable::createBallCache()
     {
         bpos += bvel;
         bvel *= bdecay;
+
+        M_ball_pos_cache.push_back( bpos );
+
+        if ( i >= 5
+             && bvel.r2() < 0.01*0.01 )
+        {
+            // ball stopped
+            break;
+        }
+
         if ( bpos.absX() > pitch_x_max
              || bpos.absY() > pitch_y_max )
         {
             // out of pitch
             break;
         }
+    }
 
-        if ( i >= 5
-             && bvel.r2() < 0.01*0.01 )
-        {
-            // ball stopped
-            M_ball_pos_cache.push_back( bpos );
-            bpos += bvel;
-            M_ball_pos_cache.push_back( bpos );
-            break;
-        }
-
+    if ( M_ball_pos_cache.size() == 1 )
+    {
         M_ball_pos_cache.push_back( bpos );
-        //M_ball_vel_cache.push_back( bvel );
     }
 }
 
@@ -387,45 +393,30 @@ InterceptTable::predictSelf()
 
     std::size_t max_cycle = std::min( MAX_CYCLE, M_ball_pos_cache.size() );
 
-    SelfIntercept predictor( M_world );
-    predictor.predict( max_cycle,
-                       M_self_cache );
+    //SelfIntercept predictor( M_world, M_ball_pos_cache );
+    SelfInterceptV13 predictor( M_world, M_ball_pos_cache );
+    predictor.predict( max_cycle, M_self_cache );
+
     if ( M_self_cache.empty() )
     {
-        std::cerr << "Interecet Self cache is empty!"
+        std::cerr << M_world.self().unum() << ' '
+                  << M_world.time()
+                  << " Interecet Self cache is empty!"
                   << std::endl;
-        // if  M_self_cache.empty() is true
+        // if self cache is empty,
         // reach point should be the inertia final point of the ball
         return;
     }
 
-#if 0
-    {
-        const std::vector< InterceptInfo >::iterator send = M_self_cache.end();
-        for ( std::vector< InterceptInfo >::iterator it = M_self_cache.begin();
-              it != send;
-              ++it )
-        {
-            dlog.addText( Logger::INTERCEPT,
-                          "== type=%d cycle=%d (turn=%d dash=%d0 dash_power=%.2f",
-                          it->mode(),
-                          it->reachCycle(),
-                          it->turnCycle(),
-                          it->dashCycle(),
-                          it->dashPower() );
-        }
-    }
-#endif
-
-#ifdef SELF_INTERCEPT_USE_NO_SAVE_RECEVERY
-    std::sort( M_self_cache.begin(),
-               M_self_cache.end(),
-               InterceptInfoCmp() );
-    M_self_cache.erase( std::unique( M_self_cache.begin(),
-                                     M_self_cache.end(),
-                                     InterceptInfoEqual() ),
-                        M_self_cache.end() );
-#endif
+// #ifdef SELF_INTERCEPT_USE_NO_SAVE_RECEVERY
+//     std::sort( M_self_cache.begin(),
+//                M_self_cache.end(),
+//                InterceptInfo::Cmp() );
+//     M_self_cache.erase( std::unique( M_self_cache.begin(),
+//                                      M_self_cache.end(),
+//                                      InterceptInfo::Equal() ),
+//                         M_self_cache.end() );
+// #endif
 
     int min_cycle = M_self_reach_cycle;
     int exhaust_min_cycle = M_self_exhaust_reach_cycle;
@@ -456,20 +447,6 @@ InterceptTable::predictSelf()
     dlog.addText( Logger::INTERCEPT,
                   "Intercept Self. solution size = %d",
                   M_self_cache.size() );
-#if 0
-    for ( std::vector< InterceptInfo >::iterator it = M_self_cache.begin();
-          it != end;
-          ++it )
-    {
-        dlog.addText( Logger::INTERCEPT,
-                      "-- type=%d cycle=%d (turn=%d dash=%d) dash_power=%.2f",
-                      it->mode(),
-                      it->reachCycle(),
-                      it->turnCycle(),
-                      it->dashCycle(),
-                      it->dashPower() );
-    }
-#endif
 
     M_self_reach_cycle = min_cycle;
     M_self_exhaust_reach_cycle = exhaust_min_cycle;
@@ -553,7 +530,11 @@ InterceptTable::predictTeammate()
                       (*it)->pos().x, (*it)->pos().y,
                       player_type->id(),
                       cycle );
-        if ( cycle < second_min_cycle )
+        if ( (*it)->goalie() )
+        {
+            M_goalie_reach_cycle = cycle;
+        }
+        else if ( cycle < second_min_cycle )
         {
             second_min_cycle = cycle;
             M_second_teammate = *it;
