@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -469,20 +469,25 @@ public:
   <pre>
   Format:
   <- (dash <power>)
+     (dash <power> <dir>)
   </pre>
 */
 class PlayerDashCommand
     : public PlayerBodyCommand {
 private:
     double M_power; //!< dash power
+    double M_dir; //!< dash direction
 public:
     /*!
       \brief construct with dash power
       \param power dash power
+      \param dir dash direction
     */
     explicit
-    PlayerDashCommand( const double & power )
+    PlayerDashCommand( const double & power,
+                       const double & dir = 0.0 )
         : M_power( power )
+        , M_dir( dir )
       { }
 
     /*!
@@ -518,6 +523,15 @@ public:
       {
           return M_power;
       }
+
+    /*!
+      \brief get dash command parameter
+      \return dash direction
+     */
+    double dashDir() const
+      {
+          return M_dir;
+      }
 };
 
 //////////////////////////////////////////////////////////////////////
@@ -717,7 +731,7 @@ public:
 
   <pre>
   Format:
-  <- (tackle <power>)
+  <- (tackle <power_or_dir>)
   </pre>
 */
 class PlayerTackleCommand
@@ -727,7 +741,7 @@ private:
 public:
     /*!
       \brief construct with tackle power
-      \param power tackle power
+      \param power_or_dir tackle power or direction
     */
     explicit
     PlayerTackleCommand( const double & power_or_dir )
