@@ -1,4 +1,4 @@
-// -*-c++-*-
+// -*-C++-*-
 
 /*!
   \file visual_sensor.cpp
@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -49,9 +49,16 @@ namespace rcsc {
 /*-------------------------------------------------------------------*/
 
 /*!
+  \struct SeenDistCmp
   \brief functor to compare seen distance of observed object
 */
 struct SeenDistCmp {
+    /*!
+      \brief operation function
+      \param lhs left hand side variable
+      \param rhs right hand side variable
+      \return compared result
+     */
     bool operator()( const VisualSensor::PolarT & lhs,
                      const VisualSensor::PolarT & rhs ) const
       {
@@ -102,6 +109,7 @@ operator<<( std::ostream & os,
        << " body=" << data.body_
        << " neck=" << data.face_
        << " point_dir=" << data.arm_
+       << " kicked=" << data.kicked_
        << " tackle=" << data.tackle_;
 
     return os;
@@ -468,9 +476,9 @@ VisualSensor::parse( const char * msg,
 
 #if 0
     dlog.addText( Logger::SENSOR,
-                  "visual.parse. : t= %d. ut= %d. o= %d. uo= %d uk= %d\n"
-                  "visual.parse.  seen markers= %d.  behind_markers= %d. lines= %d\n"
-                  " ball= %d",,
+                  __FILE__" (parse) t=%d ut=%d o=%d uo=%d uk=%d"
+                  "  seen markers=%d.  behind_markers=%d. lines=%d"
+                  " ball=%d",
                   M_teammates.size(), M_unknown_teammates.size(),
                   M_opponents.size(), M_unknown_opponents.size(),
                   M_unknown_players.size()
@@ -481,7 +489,7 @@ VisualSensor::parse( const char * msg,
     if ( M_markers.size() > 10 )
     {
         dlog.addText( Logger::SENSOR,
-                      "visual.parse. marker resize to 10 from %d",
+                      __FILE__" (parse) marker resize to 10 from %d",
                       M_markers.size() );
         M_markers.resize( 10 );
     }
@@ -872,15 +880,15 @@ VisualSensor::parsePlayer( const char * tok,
     // "tok" must point object name next space
 
     // check positional info pattern
-    // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <POINTDIR> <TACKLE>)" : space = 8
+    // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <POINTDIR> <TACKLE|KICK>)" : space = 8
     // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <POINTDIR>)" : space = 7
-    // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <TACKLE>)" : space = 7
+    // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <TACKLE|KICK>)" : space = 7
     // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD>)" : space = 6
     // " <DIST> <DIR> <DISTCH> <DIRCH> <BODY>)" : space = 5  only sserver-4
     // " <DIST> <DIR> <DISTCH> <DIRCH>)" : space = 4
-    // " <DIST> <DIR> <POINTDIR> <TACKLE>)" : space = 4
+    // " <DIST> <DIR> <POINTDIR> <TACKLE|KICK>)" : space = 4
     // " <DIST> <DIR> <POINTDIR>)" : space = 3
-    // " <DIST> <DIR> <TACKLE>)" : space = 3
+    // " <DIST> <DIR> <TACKLE|KICK>)" : space = 3
     // " <DIST> <DIR>)" : space = 2
     // " <DIR>)" : space = 1
 
@@ -905,9 +913,10 @@ VisualSensor::parsePlayer( const char * tok,
         info->dir_chng_  = std::strtod( tok, &next ); tok = next;
         info->body_ = std::strtod( tok, &next ); tok = next;
         info->face_ = std::strtod( tok, &next ); tok = next;
-        info->arm_ = std::strtod( tok, NULL );
+        info->arm_ = std::strtod( tok, &next ); tok = next;
         info->has_vel_ = true;
-        info->tackle_ = true;
+        if ( *(tok + 1) == 'k' ) info->kicked_ = true;
+        if ( *(tok + 1) == 't' ) info->tackle_ = true;
     }
     // <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <POINTDIR>
     // <DIST> <DIR> <DISTCH> <DIRCH> <BODY> <HEAD> <TACKLE>
@@ -920,7 +929,11 @@ VisualSensor::parsePlayer( const char * tok,
         info->body_ = std::strtod( tok, &next ); tok = next;
         info->face_ = std::strtod( tok, &next ); tok = next;
         info->has_vel_ = true;
-        if (*(tok + 1) == 't')
+        if ( *(tok + 1) == 'k' )
+        {
+            info->kicked_ = true;
+        }
+        else if ( *(tok + 1) == 't' )
         {
             info->tackle_ = true;
         }
@@ -958,7 +971,12 @@ VisualSensor::parsePlayer( const char * tok,
         info->dist_ = std::strtod( tok, &next ); tok = next;
         info->dir_  = std::strtod( tok, &next ); tok = next;
         double tmp = std::strtod( tok, &next ); tok = next;
-        if ( *(tok + 1) == 't' )
+        if ( *(tok + 1) == 'k' )
+        {
+            info->arm_ = tmp;
+            info->kicked_ = true;
+        }
+        else if ( *(tok + 1) == 't' )
         {
             info->arm_ = tmp;
             info->tackle_ = true;
@@ -975,7 +993,11 @@ VisualSensor::parsePlayer( const char * tok,
     {
         info->dist_ = std::strtod( tok, &next ); tok = next;
         info->dir_  = std::strtod( tok, &next ); tok = next;
-        if ( *(tok + 1) == 't' )
+        if ( *(tok + 1) == 'k' )
+        {
+            info->kicked_ = true;
+        }
+        else if ( *(tok + 1) == 't' )
         {
             info->tackle_ = true;
         }
