@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -95,5 +95,23 @@ ParserV1::parse( std::istream & is,
     return false;
 }
 
+
+/*-------------------------------------------------------------------*/
+/*!
+
+*/
+namespace {
+
+Parser::Ptr
+create_v1()
+{
+    Parser::Ptr ptr( new ParserV1() );
+    return ptr;
+}
+
+rcss::RegHolder v1 = Parser::creators().autoReg( &create_v1, REC_OLD_VERSION );
+
+}
+
 } // end of namespace
 } // end of namespace
