@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,13 +42,14 @@
 
 #include "serializer.h"
 
+#include "util.h"
+
 #include "serializer_v1.h"
 #include "serializer_v2.h"
 #include "serializer_v3.h"
 #include "serializer_v4.h"
 #include "serializer_v5.h"
 
-#include "util.h"
 
 #include <algorithm>
 #include <cstring>
@@ -73,10 +74,10 @@ namespace rcg {
 /*!
 
 */
-Serializer::FactoryHolder &
-Serializer::factoryHolder()
+Serializer::Creators &
+Serializer::creators()
 {
-    static FactoryHolder s_instance;
+    static Creators s_instance;
     return s_instance;
 }
 
@@ -88,10 +89,10 @@ Serializer::factoryHolder()
 Serializer::Ptr
 Serializer::create( const int version )
 {
-    Serializer::Ptr ptr;
+    Serializer::Ptr ptr( static_cast< Serializer * >( 0 ) );
 
     Serializer::Creator creator;
-    if ( Serializer::factoryHolder().getCreator( creator, version ) )
+    if ( Serializer::creators().getCreator( creator, version ) )
     {
         ptr = creator();
     }
