@@ -13,7 +13,7 @@
  This code is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
- version 2.1 of the License, or (at your option) any later version.
+ version 3 of the License, or (at your option) any later version.
 
  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -824,7 +824,7 @@ class TrainerChangePlayerTypeCommand
 private:
     std::string M_team_name; //!< target player's team name
     int M_unum; //!< target player's uniform number
-    HeteroID M_type; //!< player type Id
+    int M_type; //!< player type Id
 public:
     /*!
       \brief construct with target player and type id
@@ -834,7 +834,7 @@ public:
     */
     TrainerChangePlayerTypeCommand( const std::string & team_name,
                                     const int unum,
-                                    const HeteroID type );
+                                    const int type );
 
     /*!
       \brief get command type
